'-----------------------------------------------------------------------
' <copyright file="SBAStandardCode.vb" company="Microsoft">
'     Copyright (c) Microsoft Corporation.  All rights reserved.
' </copyright>
' <disclaimer>
'  THIS CODE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, WHETHER EXPRESSED
'  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
'  MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK OF
'  USE OR RESULTS IN CONNECTION WITH THE USE OF THIS CODE REMAINS WITH THE USER.  
' </disclaimer>
'-----------------------------------------------------------------------

Public NotInheritable Class SBAAppLoader

   ' The strong name of the Loader assembly
   Private Const loaderFULLNAME As String = "Loader, Version=2.0.5201.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35"
   ' The fully qualified name of the Loader class
   Private Const loaderNAMESPACE As String = "Microsoft.BusinessSolutions.SmallBusinessAccounting.Loader.Loader"

   ' The name of .resx file containing localizable UI resources
   Private Const resourceFILE As String = "SBASolution.SBAResources"
   ' Resource manager for localizable UI resources
   Friend Shared isvResources As ResourceManager = New ResourceManager(resourceFILE, Assembly.GetExecutingAssembly())

   ' ISbaObjects contains references to the entry points of the SBA object model
   Private Shared sbaObjects As ISbaObjects
   ' IConfiguration object to instantiate sBA objects in V1 app domain
   Private Shared sbcConfiguration As IConfiguration

   ' Private constructor 
   Private Sub New()

   End Sub
   ' GetSbaObjects 
   '
   ' GetSbaObjects attempts to load a specific SBC company file. If 
   ' the file isn't there, it opens an OpenFileDialog to allow the user to 
   ' choose an alternative file.
   '
   ' Once it has a valid file, it attempts to load the Loader and 
   ' use the company file to attach to a database.
   Private Shared Function LoadSbaObjects(ByVal sbcFileTemp As String) As ISbaObjects
      Dim loaderAssembly As Assembly         ' The assembly containing the Loader object
      Dim sbaLoader As ILoader               ' The Loader object
      Dim sbcConnector As IConnector         ' An IConnector instance, used to verify version and culture
      Dim sbcFile As String                  ' The path and filename of an SBA company file

      ' Define variables for message boxes
      Dim message As String
      Dim buttons As MsgBoxStyle = MsgBoxStyle.OkOnly Or MsgBoxStyle.Critical

      ' Set default path
      Dim sbcDefaultPath As String = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) & _
                       Path.DirectorySeparatorChar & _
                       "Small Business Accounting" & _
                       Path.DirectorySeparatorChar & _
                       "Companies" & _
                       Path.DirectorySeparatorChar

      ' If sbcFileTemp does not contain path information, append it to the default path
      ' Otherwise, use it as is
      If Not String.IsNullOrEmpty(sbcFileTemp) AndAlso sbcFileTemp.Contains(Path.DirectorySeparatorChar.ToString()) Then
         sbcFile = sbcFileTemp
      Else
         sbcFile = sbcDefaultPath & sbcFileTemp
      End If

      ' If sbcFile does not exist, open an OpenFileDialog
      If Not File.Exists(sbcFile) Then
         Dim openFileDialog1 As New OpenFileDialog()

         openFileDialog1.InitialDirectory = sbcDefaultPath
         openFileDialog1.Filter = isvResources.GetString("resFileOpenSbcFilter")
         openFileDialog1.FilterIndex = 1
         openFileDialog1.CheckFileExists = True
         If openFileDialog1.ShowDialog() <> DialogResult.OK Then
            Return Nothing
         End If
         sbcFile = openFileDialog1.FileName
      End If

      ' Load Loader implementation assembly from GAC
      '  (Note - Assembly.Load does not return null in normal operation)
      loaderAssembly = Assembly.Load(loaderFULLNAME)

      ' Instantiate ILoader object.  Exit if instantiation fails.
      sbaLoader = DirectCast(loaderAssembly.CreateInstance(loaderNAMESPACE), ILoader)
      If sbaLoader Is Nothing Then
         message = isvResources.GetString("resLoaderErrorLong")
         MsgBox(message, buttons, isvResources.GetString("resLoaderErrorShort"))
         Return Nothing
      End If

      ' Instantiate IConnector object for version checking
      sbcConnector = sbaLoader.GetConnector(sbcFile)

      ' Get configuration information - required to launch in a new AppDomain
      sbcConfiguration = sbcConnector.GetConfiguration()

      ' Check version and culture:
      '    Exit if the runtime version is less than the required version.
      '    Exit if there is a required culture and it is not present.
      '    Force new AppDomain if the runtime version is 1.
      runtimeMajorVersion = sbcConnector.RequiredBuildVersion.Major

      If runtimeMajorVersion < requiredMajorVersion Then
         message = isvResources.GetString("resVersionErrorPart1") & _
                   requiredMajorVersion.ToString(CultureInfo.CurrentCulture) & _
                   isvResources.GetString("resVersionErrorPart2") & _
                   runtimeMajorVersion.ToString(CultureInfo.CurrentCulture) & _
                   isvResources.GetString("resVersionErrorPart3")
         MsgBox(message, buttons, isvResources.GetString("resVersionErrorShort"))
         Return Nothing
      End If

      If runtimeMajorVersion = 1 Then createInAppDomain = True

      runtimeCulture = sbcConnector.RequiredCulture.Name
      If Not String.IsNullOrEmpty(requiredCulture) AndAlso requiredCulture.CompareTo(runtimeCulture) <> 0 Then
         message = isvResources.GetString("resCultureErrorPart1") & _
                   requiredCulture & _
                   isvResources.GetString("resCultureErrorPart2") & _
                   runtimeCulture & _
                   isvResources.GetString("resCultureErrorPart3")
         MsgBox(message, buttons, isvResources.GetString("resCultureErrorShort"))
         Return Nothing
      End If

      ' Connect to the database.  Exit if the connection fails.
      Try
         sbaObjects = sbaLoader.GetSbaObjects(sbcFile)
         If sbaObjects Is Nothing OrElse sbaObjects.SmallBusinessInstance Is Nothing Then
            message = isvResources.GetString("resDBErrorLong")
            MsgBox(message, buttons, isvResources.GetString("resDBErrorShort"))
            Return Nothing
         End If
      Catch e As ApplicationException
         If TypeOf e Is ILoaderException Then
            message = isvResources.GetString("resDBErrorLong")
            MsgBox(message, buttons, isvResources.GetString("resDBErrorShort"))
            Return Nothing
         Else
            Throw
         End If
      End Try

      ' Return the connected sbaObjects instance.
      Return sbaObjects
   End Function

   ' GetSBAInstallationLocation()
   '
   '  Defines the base directory of the new AppDomain.
   '
   '  Applications that connect to a V1 installation of Small Business Accounting
   '  should run in an AppDomain whose search path includes the directory in which
   '  SBA is installed.  This method retrieves that path from the Registry.
   Private Shared Function GetSBAInstallationLocation() As String
      Const keyPATH As String = "SOFTWARE\Microsoft\Business Solutions eCRM\2.0\"

      Dim key As RegistryKey = Nothing
      Dim installLocation As String = String.Empty

      Try
         key = Registry.LocalMachine.OpenSubKey(keyPATH)
         If key IsNot Nothing Then
            Dim regPath As String = CStr(key.GetValue("AccountingUI"))
            installLocation = Path.GetDirectoryName(Path.GetDirectoryName(regPath))
         End If
         ' TODO: Provide default value (Program Files? or just return null?)
      Catch ex As ArgumentException
         ' TODO: Handle argument exception

      Catch ex As SecurityException
         ' TODO: Handle security exception (Registry read permission denied)

      Finally
         If key IsNot Nothing Then key.Close()
      End Try

      Return installLocation
   End Function

   Private Shared Sub RunISVApplication(ByVal sbaFormType As Type, ByVal sbaBaseDirectory As String, ByVal sbaSearchPath As String)

      If createInAppDomain Then
         ' Launch an SBAForm in a new AppDomain (required for version1)

         ' create shared AppDomain with solution and Small Business Accounting                
         Dim isvDomain As AppDomain = AppDomain.CreateDomain("V1Domain", Nothing, sbaBaseDirectory, sbaSearchPath, False)

         ' Extract database name and server name so they can be serialized and passed
         '  in to the SBAForm constructor
         Dim args() As Object = {sbcConfiguration.DatabaseName, sbcConfiguration.ServerName}

         ' create instance of form launcher in new appdomain
            Dim sbaFormInst As Object = isvDomain.CreateInstanceAndUnwrap(Assembly.GetExecutingAssembly().FullName, _
                                                sbaFormType.FullName, _
                                                True, _
                                                BindingFlags.CreateInstance, _
                                                Nothing, _
                                                args, _
                                                Nothing, _
                                                Nothing, _
                                                Nothing)
         ' launch the form
         sbaFormType.InvokeMember("Run", BindingFlags.InvokeMethod, Nothing, sbaFormInst, Nothing, System.Globalization.CultureInfo.CurrentCulture)
      Else
         ' Launch an SBAForm in the current AppDomain

         ' Create arrays describing sbaObjects
         Dim constructorParamTypes() As Type = {CObj(sbaObjects).GetType()}
         Dim constructorParams() As Object = {sbaObjects}

         ' Get and invoke the form constructor
         Dim sbaConstructor As ConstructorInfo = sbaFormType.GetConstructor(constructorParamTypes)
         Dim sbaFormInst As Object = sbaConstructor.Invoke(constructorParams)

         ' Launch the form
         sbaFormType.InvokeMember("Run", BindingFlags.InvokeMethod, Nothing, sbaFormInst, Nothing, System.Globalization.CultureInfo.CurrentCulture)
      End If
   End Sub
End Class
